# Customer ID=7729; Build=0x71521; Copyright (c) 2013 by Cadence Design Systems Inc.  ALL RIGHTS RESERVED.
# These coded instructions, statements, and computer programs
# are the copyrighted works and confidential proprietary information of
# Cadence Design Systems Inc.  They may not be modified, copied,
# reproduced, distributed, or disclosed to third parties in any manner,
# medium, or form, in whole or in part, without the prior written
# consent of Cadence Design Systems Inc.

package XtensaPorts;

use strict;
use Carp;
use Exporter;
@XtensaPorts::ISA = qw(Exporter);
@XtensaPorts::EXPORT = qw();
@XtensaPorts::EXPORT_OK = ();
@XtensaPorts::EXPORT_TAGS = ();
use vars qw(%parameters %ports);


my %buildoptions = (
    'trax'              => 1,
    'trax_log_mem_size' => 12,
);

sub getBuildOptions {
   sort keys %buildoptions;
}

sub getBuildOption {
    my $option = shift;
    die "Error: option $option doesn't exist in XtensaPorts\n" unless(exists $buildoptions{$option});
    $buildoptions{$option};
}

%parameters = (
    'maxiwidth'     => 64,
    'width'         => 32,
    'iwidth'        => 64,
    'ridx'          => 4,
    'pcwidth'       => 32,
    'awidth'        => 32,
    'eawidth'       => 32,
    'pawidth'       => 32,
    'eintrwidth'    => 27,
    'bewidth'       => 8,
    'iccntsize'     => 4,
    'icwidth'       => 64,
    'iccwidth'      => 0,
    'itwidth'       => 21,
    'itcwidth'      => 0,
    'wordenwidth'   => 2,
    'iramwidth'     => 64,
    'iramwords'     => 2,
    'iramawidth'    => 12,
    'iram0width'    => 64,
    'iram0words'    => 2,
    'iram0awidth'   => 12,
    'iram0cwidth'   => 0,
    'iram1width'    => 64,
    'iram1words'    => 2,
    'iram1awidth'   => 11,
    'iram1cwidth'   => 0,
    'iromwidth'     => 0,
    'iromwords'     => 0,
    'asize'         => 6,
    'tpdwidth'      => 32,
    'tpiwidth'      => 32,
    'tpswidth'      => 8,
    'dwaybawidth'   => 14,
    'dcwidth'       => 64,
    'nbytes'        => 8,
    'dccwidth'      => 0,
    'dtwidth'       => 22,
    'dtcwidth'      => 0,
    'dcwidthoff'    => 3,
    'dclineoffset'  => 7,
    'edrdwidth'     => 64,
    'edwrwidth'     => 64,
    'edrdbytecnt'   => 8,
    'edrdlbw'       => 3,
    'jtiwidth'      => 5,
    'cpn'           => 2,
    'iPgLsb'        => 29,
    'dPgLsb'        => 29,
    'itawidth'      => 7,
    'icsetawidth'   => 11,
    'iromawidth'    => 0,
    'wbcountwidth'  => 5,
    'reqid'         => 6,
    'dreqid'        => 6,
    'ifwayselwidth' => 3,
   );
sub getParameters {
   \%parameters;
}
# Port info structure 
# signal_name -> [ direction, bit_index, group, interface_type, protocol, delay, isolation ] 
%ports = (
    'CLK'                => [ 'input',         '', 'Global', 'xtintfGlobal', 'CLK', '', '' ],
    'BReset'             => [ 'input',         '', 'Global', 'xtintfGlobal', 'Reset', 'C*0.25', '-' ],
    'ICacheAData'        => [ 'input',   '[63:0]', 'ICache', 'xtintfInstCache', 'CacheAData', '0.5511', '-' ],
    'ITagAData'          => [ 'input',   '[20:0]', 'ICache', 'xtintfInstCache', 'TagAData', '0.3729', '-' ],
    'ICacheBData'        => [ 'input',   '[63:0]', 'ICache', 'xtintfInstCache', 'CacheBData', '0.5511', '-' ],
    'ITagBData'          => [ 'input',   '[20:0]', 'ICache', 'xtintfInstCache', 'TagBData', '0.3729', '-' ],
    'ICacheCData'        => [ 'input',   '[63:0]', 'ICache', 'xtintfInstCache', 'CacheCData', '0.5511', '-' ],
    'ITagCData'          => [ 'input',   '[20:0]', 'ICache', 'xtintfInstCache', 'TagCData', '0.3729', '-' ],
    'ICacheDData'        => [ 'input',   '[63:0]', 'ICache', 'xtintfInstCache', 'CacheDData', '0.5511', '-' ],
    'ITagDData'          => [ 'input',   '[20:0]', 'ICache', 'xtintfInstCache', 'TagDData', '0.3729', '-' ],
    'IRam0Data'          => [ 'input',   '[63:0]', 'IRam0', 'xtintfInstRam', 'Data', '0.627', 'M0' ],
    'IRam1Data'          => [ 'input',   '[63:0]', 'IRam1', 'xtintfInstRam', 'Data', '0.5511', 'M0' ],
    'DTagAData0'         => [ 'input',   '[21:0]', 'DCache0', 'xtintfDataCache', 'TagAData', '0.3729', '-' ],
    'DTagBData0'         => [ 'input',   '[21:0]', 'DCache0', 'xtintfDataCache', 'TagBData', '0.3729', '-' ],
    'DTagCData0'         => [ 'input',   '[21:0]', 'DCache0', 'xtintfDataCache', 'TagCData', '0.3729', '-' ],
    'DTagDData0'         => [ 'input',   '[21:0]', 'DCache0', 'xtintfDataCache', 'TagDData', '0.3729', '-' ],
    'DCacheAData0'       => [ 'input',   '[63:0]', 'DCache0', 'xtintfDataCache', 'CacheAData', '0.5511', '-' ],
    'DCacheBData0'       => [ 'input',   '[63:0]', 'DCache0', 'xtintfDataCache', 'CacheBData', '0.5511', '-' ],
    'DCacheCData0'       => [ 'input',   '[63:0]', 'DCache0', 'xtintfDataCache', 'CacheCData', '0.5511', '-' ],
    'DCacheDData0'       => [ 'input',   '[63:0]', 'DCache0', 'xtintfDataCache', 'CacheDData', '0.5511', '-' ],
    'DRam0Data0'         => [ 'input',   '[63:0]', 'DRam0', 'xtintfDataRam', 'Data', '0.627', '-' ],
    'DRam1Data0'         => [ 'input',   '[63:0]', 'DRam1', 'xtintfDataRam', 'Data', '0.9086', '-' ],
    'PIReqRdy'           => [ 'input',         '', 'PIFM', 'xtintfPIF', 'ReqRdy', 'C*0.60', '-' ],
    'PIRespData'         => [ 'input',   '[63:0]', 'PIFM', 'xtintfPIF', 'RespData', 'C*0.70', '-' ],
    'PIRespCntl'         => [ 'input',    '[7:0]', 'PIFM', 'xtintfPIF', 'RespCntl', 'C*0.70', '-' ],
    'PIRespValid'        => [ 'input',         '', 'PIFM', 'xtintfPIF', 'RespValid', 'C*0.70', '-' ],
    'PIRespPriority'     => [ 'input',    '[1:0]', 'PIFM', 'xtintfPIF', 'RespPriority', 'C*0.70', '-' ],
    'PIRespId'           => [ 'input',    '[5:0]', 'PIFM', 'xtintfPIF', 'RespId', 'C*0.70', '-' ],
    'PIReqValid'         => [ 'input',         '', 'PIFS', 'xtintfPIF', 'ReqValid', 'C*0.70', '-' ],
    'PIReqCntl'          => [ 'input',    '[7:0]', 'PIFS', 'xtintfPIF', 'ReqCntl', 'C*0.70', '-' ],
    'PIReqAdrs'          => [ 'input',   '[31:0]', 'PIFS', 'xtintfPIF', 'ReqAdrs', 'C*0.70', '-' ],
    'PIReqData'          => [ 'input',   '[63:0]', 'PIFS', 'xtintfPIF', 'ReqData', 'C*0.70', '-' ],
    'PIReqDataBE'        => [ 'input',    '[7:0]', 'PIFS', 'xtintfPIF', 'ReqDataBE', 'C*0.70', '-' ],
    'PIReqPriority'      => [ 'input',    '[1:0]', 'PIFS', 'xtintfPIF', 'ReqPriority', 'C*0.70', '-' ],
    'PIReqId'            => [ 'input',    '[5:0]', 'PIFS', 'xtintfPIF', 'ReqId', 'C*0.70', '-' ],
    'PIRespRdy'          => [ 'input',         '', 'PIFS', 'xtintfPIF', 'RespRdy', 'C*0.70', '-' ],
    'PrefetchRamData'    => [ 'input',  '[127:0]', 'Prefetch', 'xtintPrefetch', 'Data', '0.407', '-' ],
    'BInterrupt'         => [ 'input',   '[26:0]', 'Interrupt', 'xtintfWire', 'Data', 'C*0.40', '-' ],
    'PRID'               => [ 'input',   '[15:0]', 'PRID', 'xtintfWire', 'Data', 'C*0.25', '-' ],
    'StatVectorSel'      => [ 'input',         '', 'Unclassified', '', '', 'C*0.25', '-' ],
    'PDebugEnable'       => [ 'input',         '', 'TracePort', 'xtintfTracePort', 'DebugEnable', 'C*0.80', 'D0' ],
    'PerfMonInt'         => [ 'input',         '', 'PerfMon', 'xtintfPerfMon', 'PerfMonInt', 'C*0.80', '-' ],
    'RunStall'           => [ 'input',         '', 'RunStall', 'xtintfWire', 'Data', 'C*0.80', '-' ],
    'TMode'              => [ 'input',         '', 'TMode', 'xtintfWire', 'Data', 'C*0.10', '-' ],
    'TModeClkGateOverride' => [ 'input',         '', 'TMode', 'xtintfWire', 'Data', 'C*0.10', '-' ],
    'OCDRSRBus_M'        => [ 'input',   '[31:0]', 'OCD', 'xtintfOCD', 'OCDRSRBus_M', 'C*0.7', 'D0' ],
    'OCDEnabled_W'       => [ 'input',         '', 'OCD', 'xtintfOCD', 'OCDEnabled_W', 'C*0.7', 'D0' ],
    'OCDOverride'        => [ 'input',         '', 'OCD', 'xtintfOCD', 'OCDOverride', 'C*0.7', 'D0' ],
    'InterruptAllConds'  => [ 'input',         '', 'OCD', 'xtintfOCD', 'InterruptAllConds', 'C*0.7', 'D0' ],
    'OCDDbgInt'          => [ 'input',         '', 'OCD', 'xtintfOCD', 'OCDDbgInt', 'C*0.7', 'D0' ],
    'ExecDebugInstr'     => [ 'input',         '', 'OCD', 'xtintfOCD', 'ExecDebugInstr', 'C*0.7', 'D0' ],
    'DebugInstr'         => [ 'input',   '[63:0]', 'OCD', 'xtintfOCD', 'DebugInstr', 'C*0.7', 'D0' ],
    'EnableRunStall'     => [ 'input',         '', 'OCD', 'xtintfOCD', 'EnableRunStall', 'C*0.7', 'D1' ],
    'DebugMode'          => [ 'input',         '', 'OCD', 'xtintfOCD', 'DebugMode', 'C*0.7', 'D0' ],
    'TIE_ERI_RD_In'      => [ 'input',   '[31:0]', 'ERI_RD', 'xtintfLookup', 'ERI_RD__DataIn', 'C*0.3', '-' ],
    'TIE_ERI_RD_Rdy'     => [ 'input',         '', 'ERI_RD', 'xtintfLookup', 'ERI_RD__Rdy', 'C*0.5', '-' ],
    'TIE_ERI_WR_In'      => [ 'input',         '', 'ERI_WR', 'xtintfLookup', 'ERI_WR__DataIn', 'C*0.3', '-' ],
    'TIE_IMPWIRE'        => [ 'input',   '[31:0]', 'IMPWIRE', 'xtintfWire', 'IMPWIRE__Data', 'C*0.7', '-' ],
    'ICacheAEn'          => [ 'output',         '', 'ICache', 'xtintfInstCache', 'CacheAEn', '0.0968', 'P0' ],
    'ICacheAWordEn'      => [ 'output',    '[1:0]', 'ICache', 'xtintfInstCache', 'CacheAWordEn', '0.0638', 'P0' ],
    'ICacheAWr'          => [ 'output',         '', 'ICache', 'xtintfInstCache', 'CacheAWr', '0.0759', 'P0' ],
    'ITagAEn'            => [ 'output',         '', 'ICache', 'xtintfInstCache', 'TagAEn', '0.0968', 'P0' ],
    'ITagAWr'            => [ 'output',         '', 'ICache', 'xtintfInstCache', 'TagAWr', '0.0539', 'P0' ],
    'ICacheBEn'          => [ 'output',         '', 'ICache', 'xtintfInstCache', 'CacheBEn', '0.0968', 'P0' ],
    'ICacheBWordEn'      => [ 'output',    '[1:0]', 'ICache', 'xtintfInstCache', 'CacheBWordEn', '0.0638', 'P0' ],
    'ICacheBWr'          => [ 'output',         '', 'ICache', 'xtintfInstCache', 'CacheBWr', '0.0759', 'P0' ],
    'ITagBEn'            => [ 'output',         '', 'ICache', 'xtintfInstCache', 'TagBEn', '0.0968', 'P0' ],
    'ITagBWr'            => [ 'output',         '', 'ICache', 'xtintfInstCache', 'TagBWr', '0.0539', 'P0' ],
    'ICacheCEn'          => [ 'output',         '', 'ICache', 'xtintfInstCache', 'CacheCEn', '0.0968', 'P0' ],
    'ICacheCWordEn'      => [ 'output',    '[1:0]', 'ICache', 'xtintfInstCache', 'CacheCWordEn', '0.0638', 'P0' ],
    'ICacheCWr'          => [ 'output',         '', 'ICache', 'xtintfInstCache', 'CacheCWr', '0.0759', 'P0' ],
    'ITagCEn'            => [ 'output',         '', 'ICache', 'xtintfInstCache', 'TagCEn', '0.0968', 'P0' ],
    'ITagCWr'            => [ 'output',         '', 'ICache', 'xtintfInstCache', 'TagCWr', '0.0539', 'P0' ],
    'ICacheDEn'          => [ 'output',         '', 'ICache', 'xtintfInstCache', 'CacheDEn', '0.0968', 'P0' ],
    'ICacheDWordEn'      => [ 'output',    '[1:0]', 'ICache', 'xtintfInstCache', 'CacheDWordEn', '0.0638', 'P0' ],
    'ICacheDWr'          => [ 'output',         '', 'ICache', 'xtintfInstCache', 'CacheDWr', '0.0759', 'P0' ],
    'ITagDEn'            => [ 'output',         '', 'ICache', 'xtintfInstCache', 'TagDEn', '0.0968', 'P0' ],
    'ITagDWr'            => [ 'output',         '', 'ICache', 'xtintfInstCache', 'TagDWr', '0.0539', 'P0' ],
    'ICacheWrData'       => [ 'output',   '[63:0]', 'ICache', 'xtintfInstCache', 'CacheWrData', '0.0638', 'P0' ],
    'ITagWrData'         => [ 'output',   '[20:0]', 'ICache', 'xtintfInstCache', 'TagWrData', '0.0693', 'P0' ],
    'ITagAddr'           => [ 'output',    '[6:0]', 'ICache', 'xtintfInstCache', 'TagAddr', '0.0495', 'P0' ],
    'ICacheAddr'         => [ 'output',   '[10:0]', 'ICache', 'xtintfInstCache', 'CacheAddr', '0.1012', 'P0' ],
    'IRam0Addr'          => [ 'output',   '[14:3]', 'IRam0', 'xtintfInstRam', 'Addr', '0.154', 'P0' ],
    'IRam0En'            => [ 'output',         '', 'IRam0', 'xtintfInstRam', 'En', '0.0979', 'P0' ],
    'IRam0Wr'            => [ 'output',         '', 'IRam0', 'xtintfInstRam', 'Wr', '0.1012', 'P0' ],
    'IRam0WordEn'        => [ 'output',    '[1:0]', 'IRam0', 'xtintfInstRam', 'WordEn', '0.0891', 'P0' ],
    'IRam0WrData'        => [ 'output',   '[63:0]', 'IRam0', 'xtintfInstRam', 'WrData', '0.0891', 'P0' ],
    'IRam0LoadStore'     => [ 'output',         '', 'IRam0', 'xtintfInstRam', 'LoadStore', 'C*0.30', 'XP0' ],
    'IRam1Addr'          => [ 'output',   '[13:3]', 'IRam1', 'xtintfInstRam', 'Addr', '0.1012', 'P0' ],
    'IRam1En'            => [ 'output',         '', 'IRam1', 'xtintfInstRam', 'En', '0.0968', 'P0' ],
    'IRam1Wr'            => [ 'output',         '', 'IRam1', 'xtintfInstRam', 'Wr', '0.0759', 'P0' ],
    'IRam1WordEn'        => [ 'output',    '[1:0]', 'IRam1', 'xtintfInstRam', 'WordEn', '0.0638', 'P0' ],
    'IRam1WrData'        => [ 'output',   '[63:0]', 'IRam1', 'xtintfInstRam', 'WrData', '0.0638', 'P0' ],
    'IRam1LoadStore'     => [ 'output',         '', 'IRam1', 'xtintfInstRam', 'LoadStore', 'C*0.30', 'XP0' ],
    'DCacheAddr0'        => [ 'output',   '[13:3]', 'DCache0', 'xtintfDataCache', 'CacheAddr', '0.1012', 'P0' ],
    'DTagAddr0'          => [ 'output',   '[13:7]', 'DCache0', 'xtintfDataCache', 'TagAddr', '0.0495', 'P0' ],
    'DTagWrData0'        => [ 'output',   '[21:0]', 'DCache0', 'xtintfDataCache', 'TagWrData', '0.0693', 'P0' ],
    'DTagAEn0'           => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'TagAEn', '0.0968', 'P0' ],
    'DTagAWr0'           => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'TagAWr', '0.0539', 'P0' ],
    'DTagBEn0'           => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'TagBEn', '0.0968', 'P0' ],
    'DTagBWr0'           => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'TagBWr', '0.0539', 'P0' ],
    'DTagCEn0'           => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'TagCEn', '0.0968', 'P0' ],
    'DTagCWr0'           => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'TagCWr', '0.0539', 'P0' ],
    'DTagDEn0'           => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'TagDEn', '0.0968', 'P0' ],
    'DTagDWr0'           => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'TagDWr', '0.0539', 'P0' ],
    'DCacheWrData0'      => [ 'output',   '[63:0]', 'DCache0', 'xtintfDataCache', 'CacheWrData', '0.0638', 'P0' ],
    'DCacheByteEn0'      => [ 'output',    '[7:0]', 'DCache0', 'xtintfDataCache', 'CacheByteEn', '0.0638', 'P0' ],
    'DCacheAEn0'         => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'CacheAEn', '0.0968', 'P0' ],
    'DCacheAWr0'         => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'CacheAWr', '0.0759', 'P0' ],
    'DCacheBEn0'         => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'CacheBEn', '0.0968', 'P0' ],
    'DCacheBWr0'         => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'CacheBWr', '0.0759', 'P0' ],
    'DCacheCEn0'         => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'CacheCEn', '0.0968', 'P0' ],
    'DCacheCWr0'         => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'CacheCWr', '0.0759', 'P0' ],
    'DCacheDEn0'         => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'CacheDEn', '0.0968', 'P0' ],
    'DCacheDWr0'         => [ 'output',         '', 'DCache0', 'xtintfDataCache', 'CacheDWr', '0.0759', 'P0' ],
    'DRam0Lock0'         => [ 'output',         '', 'DRam0', 'xtintfDataRam', 'Lock', '0.154', 'P0' ],
    'DRam0Addr0'         => [ 'output',   '[14:3]', 'DRam0', 'xtintfDataRam', 'Addr', '0.154', 'P0' ],
    'DRam0En0'           => [ 'output',         '', 'DRam0', 'xtintfDataRam', 'En', '0.0979', 'P0' ],
    'DRam0ByteEn0'       => [ 'output',    '[7:0]', 'DRam0', 'xtintfDataRam', 'ByteEn', '0.0891', 'P0' ],
    'DRam0Wr0'           => [ 'output',         '', 'DRam0', 'xtintfDataRam', 'Wr', '0.1012', 'P0' ],
    'DRam0WrData0'       => [ 'output',   '[63:0]', 'DRam0', 'xtintfDataRam', 'WrData', '0.0891', 'P0' ],
    'DRam1Lock0'         => [ 'output',         '', 'DRam1', 'xtintfDataRam', 'Lock', '0.154', 'P0' ],
    'DRam1Addr0'         => [ 'output',   '[16:3]', 'DRam1', 'xtintfDataRam', 'Addr', '0.154', 'P0' ],
    'DRam1En0'           => [ 'output',         '', 'DRam1', 'xtintfDataRam', 'En', '0.0968', 'P0' ],
    'DRam1ByteEn0'       => [ 'output',    '[7:0]', 'DRam1', 'xtintfDataRam', 'ByteEn', '0.1089', 'P0' ],
    'DRam1Wr0'           => [ 'output',         '', 'DRam1', 'xtintfDataRam', 'Wr', '0.0935', 'P0' ],
    'DRam1WrData0'       => [ 'output',   '[63:0]', 'DRam1', 'xtintfDataRam', 'WrData', '0.1089', 'P0' ],
    'POReqValid'         => [ 'output',         '', 'PIFM', 'xtintfPIF', 'ReqValid', 'C*0.80', '-' ],
    'POReqCntl'          => [ 'output',    '[7:0]', 'PIFM', 'xtintfPIF', 'ReqCntl', 'C*0.80', '-' ],
    'POReqAdrs'          => [ 'output',   '[31:0]', 'PIFM', 'xtintfPIF', 'ReqAdrs', 'C*0.80', '-' ],
    'POReqData'          => [ 'output',   '[63:0]', 'PIFM', 'xtintfPIF', 'ReqData', 'C*0.80', '-' ],
    'POReqDataBE'        => [ 'output',    '[7:0]', 'PIFM', 'xtintfPIF', 'ReqDataBE', 'C*0.80', '-' ],
    'POReqPriority'      => [ 'output',    '[1:0]', 'PIFM', 'xtintfPIF', 'ReqPriority', 'C*0.80', '-' ],
    'POReqId'            => [ 'output',    '[5:0]', 'PIFM', 'xtintfPIF', 'ReqId', 'C*0.80', '-' ],
    'PORespRdy'          => [ 'output',         '', 'PIFM', 'xtintfPIF', 'RespRdy', 'C*0.50', '-' ],
    'POReqRdy'           => [ 'output',         '', 'PIFS', 'xtintfPIF', 'ReqRdy', 'C*0.80', '-' ],
    'PORespData'         => [ 'output',   '[63:0]', 'PIFS', 'xtintfPIF', 'RespData', 'C*0.80', '-' ],
    'PORespCntl'         => [ 'output',    '[7:0]', 'PIFS', 'xtintfPIF', 'RespCntl', 'C*0.80', '-' ],
    'PORespValid'        => [ 'output',         '', 'PIFS', 'xtintfPIF', 'RespValid', 'C*0.80', '-' ],
    'PORespPriority'     => [ 'output',    '[1:0]', 'PIFS', 'xtintfPIF', 'RespPriority', 'C*0.80', '-' ],
    'PORespId'           => [ 'output',    '[5:0]', 'PIFS', 'xtintfPIF', 'RespId', 'C*0.80', '-' ],
    'PrefetchRamEn'      => [ 'output',         '', 'Prefetch', 'xtintPrefetch', 'En', '0.0968', '-' ],
    'PrefetchRamPIFWEn'  => [ 'output',    '[1:0]', 'Prefetch', 'xtintPrefetch', 'PIFWEn', '0.0968', '-' ],
    'PrefetchRamWr'      => [ 'output',         '', 'Prefetch', 'xtintPrefetch', 'Wr', '0.0539', '-' ],
    'PrefetchRamAddr'    => [ 'output',    '[5:0]', 'Prefetch', 'xtintPrefetch', 'Addr', '0.0495', '-' ],
    'PrefetchRamWrData'  => [ 'output',  '[127:0]', 'Prefetch', 'xtintPrefetch', 'WrData', '0.0594', '-' ],
    'PDebugLS0Stat'      => [ 'output',   '[31:0]', 'TracePort', 'xtintfTracePort', 'DebugLS', 'C*0.20', 'P0' ],
    'PDebugOutPif'       => [ 'output',    '[7:0]', 'TracePort', 'xtintfTracePort', 'DebugPif', 'C*0.20', 'P0' ],
    'PDebugInbPif'       => [ 'output',    '[7:0]', 'TracePort', 'xtintfTracePort', 'DebugPif', 'C*0.20', 'P0' ],
    'PDebugPrefetchLookup' => [ 'output',    '[7:0]', 'TracePort', 'xtintfTracePort', 'DebugPif', 'C*0.20', 'P0' ],
    'PDebugPrefetchL1Fill' => [ 'output',    '[3:0]', 'TracePort', 'xtintfTracePort', 'DebugPif', 'C*0.20', 'P0' ],
    'PDebugData'         => [ 'output',   '[31:0]', 'TracePort', 'xtintfTracePort', 'DebugData', 'C*0.20', 'P0' ],
    'PDebugInst'         => [ 'output',   '[31:0]', 'TracePort', 'xtintfTracePort', 'DebugInst', 'C*0.20', 'P0' ],
    'PDebugStatus'       => [ 'output',    '[7:0]', 'TracePort', 'xtintfTracePort', 'DebugStatus', 'C*0.20', 'P0' ],
    'PDebugPC'           => [ 'output',   '[31:0]', 'TracePort', 'xtintfTracePort', 'DebugPC', 'C*0.20', 'P0' ],
    'PWaitMode'          => [ 'output',         '', 'Interrupt', '', 'WaitMode', 'C*0.85', 'XP1' ],
    'GlobalStall'        => [ 'output',         '', 'OCD', 'xtintfOCD', 'GlobalStall', 'C*0.7', 'P0' ],
    'Valid_W'            => [ 'output',         '', 'OCD', 'xtintfOCD', 'Valid_W', 'C*0.7', 'P0' ],
    'RealExcept_W'       => [ 'output',         '', 'OCD', 'xtintfOCD', 'RealExcept_W', 'C*0.7', 'P0' ],
    'RFDO_W'             => [ 'output',         '', 'OCD', 'xtintfOCD', 'RFDO_W', 'C*0.7', 'P0' ],
    'RFDD_W'             => [ 'output',         '', 'OCD', 'xtintfOCD', 'RFDD_W', 'C*0.7', 'P0' ],
    'ReceivedRunStall'   => [ 'output',         '', 'OCD', 'xtintfOCD', 'ReceivedRunStall', 'C*0.7', 'P0' ],
    'InOCDMode_ps_C3'    => [ 'output',         '', 'OCD', 'xtintfOCD', 'InOCDMode_ps_C3', 'C*0.7', 'P0' ],
    'Interrupt5_1hot_EXC' => [ 'output',         '', 'OCD', 'xtintfOCD', 'Interrupt5_1hot_EXC', 'C*0.7', 'P0' ],
    'PSCurIntLvl_W'      => [ 'output',    '[3:0]', 'OCD', 'xtintfOCD', 'PSCurIntLvl_W', 'C*0.7', 'P0' ],
    'WSRBus_W'           => [ 'output',   '[31:0]', 'OCD', 'xtintfOCD', 'WSRBus_W', 'C*0.7', 'P0' ],
    'SRWrite_W'          => [ 'output',         '', 'OCD', 'xtintfOCD', 'SRWrite_W', 'C*0.7', 'P0' ],
    'SRAddr_W'           => [ 'output',    '[7:0]', 'OCD', 'xtintfOCD', 'SRAddr_W', 'C*0.7', 'P0' ],
    'SRRead_W'           => [ 'output',         '', 'OCD', 'xtintfOCD', 'SRRead_W', 'C*0.7', 'P0' ],
    'TIE_ERI_RD_Out'     => [ 'output',   '[13:0]', 'ERI_RD', 'xtintfLookup', 'ERI_RD__DataOut', 'C*0.3', 'XP0' ],
    'TIE_ERI_RD_Out_Req' => [ 'output',         '', 'ERI_RD', 'xtintfLookup', 'ERI_RD__Req', 'C*0.7', 'XP0' ],
    'TIE_ERI_WR_Out'     => [ 'output',   '[45:0]', 'ERI_WR', 'xtintfLookup', 'ERI_WR__DataOut', 'C*0.3', 'XP0' ],
    'TIE_ERI_WR_Out_Req' => [ 'output',         '', 'ERI_WR', 'xtintfLookup', 'ERI_WR__Req', 'C*0.7', 'XP0' ],
    'TIE_EXPSTATE'       => [ 'output',   '[31:0]', 'EXPSTATE', 'xtintfWire', 'EXPSTATE__Data', 'C*0.7', 'XP0' ],
    'EncodedExceptionVector_W' => [ 'output',    '[4:0]', 'FaultInterface', '', 'EncodedExceptionVector_W', 'C*0.2', 'P0' ],
   );

sub getGroups {
    my %groups;
    foreach (keys %ports) {
        next if($ports{$_}->[2] eq 'Global');
        $groups{$ports{$_}->[2]}++;
    }
    sort keys %groups;
}
sub getGroupsSignals {
    my ($group) = @_;
    my %signals;
    foreach (keys %ports) {
        $signals{$_}++ if($ports{$_}->[2] eq $group);
    }
    sort keys %signals;
}
sub getGroupsOutputs {
    my ($group) = @_;
    my %signals;
    foreach (keys %ports) {
        $signals{$_}++ if(($ports{$_}->[2] eq $group) && ($ports{$_}->[0] eq 'output'));
    }
    sort keys %signals;
}
sub getGroupsInputs {
    my ($group) = @_;
    my %signals;
    foreach (keys %ports) {
        $signals{$_}++ if(($ports{$_}->[2] eq $group) && ($ports{$_}->[0] eq 'input'));
    }
    sort keys %signals;
}
sub getSignalsDirection {
    my ($signal) = @_;
    $ports{$signal}->[0];
}
sub getDir {
    getSignalsDirection(@_)
}
sub getSignalsBitIndex {
    my ($signal) = @_;
    $ports{$signal}->[1];
}
sub getSignalsWidth {
    my ($signal) = @_;
    croak "Error: cannot find signal $signal\n" if(!exists $ports{$signal});
    if ($ports{$signal}->[1] eq "") {
        return 1;
    } elsif ($ports{$signal}->[1] =~ /\A \[? (\d+) : (\d+) \]? \z/xms) {
        my ($msb, $lsb) = ($1, $2);
        return($msb - $lsb + 1);
    } else {
	 croak "Error: unrecognized bit index string: $ports{$signal}->[1] \n";
    }
}
sub getSignalsGroup {
    my ($signal) = @_;
    $ports{$signal}->[2];
}
sub getPorts {
    my %remove_dir;
    foreach (keys %ports) {
       $remove_dir{$_} = $ports{$_}->[1];
    }
    \%remove_dir;
}
sub getSignalsInterfaceType {
    my ($signal) = @_;
    $ports{$signal}->[3];
}
sub getSignalsProtocol {
    my ($signal) = @_;
    $ports{$signal}->[4];
}
sub getSignalsDelay {
    my ($signal) = @_;
    $ports{$signal}->[5];
}
sub getSignalsIsoVal {
    my ($signal) = @_;
    $ports{$signal}->[6];
}
sub getIOC {
    my %ConsHash;
    foreach my $group ('Global', getGroups) {
        foreach my $signal (getGroupsSignals($group)) {
            next if($signal =~ /CLK$/);
            next if($signal eq 'JTCK');
            next if($signal eq 'JTDI');
            (my $dir = getDir($signal)) =~ s/^([io]).+$/$1/;
            (my $timing = getSignalsDelay($signal)) =~ s/C\*/Target_ClockPeriod \* /;
            my $clock = ($group =~ /^PIF[MS]$/) ? 'BCLK' :
                        ($group eq 'TAP')       ? 'JTCK' :
                        ($group eq 'APB')       ? 'PBCLK' :
                                                  'CLK';
	    $ConsHash{$group}{$signal} = {dir => $dir, clock => $clock, timing => $timing};
        }
    }
    return %ConsHash;
}
1;

